/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
**+
** NAME:
**     widtools
**
** PURPOSE:
**     Widget tools for use in shell scripts
**     Execute the program in your script with the $() construct
**
** INPUTS:
**     Execute widtools with no arguments for a list of valid inputs 
**   
** EXAMPLE:
**     #!/bin/sh
**     ans=$(widtools -dialog -type question -label "Really Quit?")
**
** RESTRICTIONS:
**     Minimal error checking is done.  If you call widtools incorrectly,
**     a segmentation fault may occur.
**
** MODIFICATION HISTORY:
**      v2.0b: RSM, 1997 MAR: widtools totally restructured
**	v1.0 : 1996 AUG, written, Robert S. Mallozzi
**                       <mallozzir@cspar.uah.edu>
**                       <http://cspar.uah.edu/~mallozzir/>
**-
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "forms.h"

#include "error.xpm"
#include "info.xpm"
#include "message.xpm"
#include "question.xpm"
#include "warning.xpm"

#define MAX_BUTTONS 100
 
void DialogCB();
void InputCB();
void MenuCB();
void QuitCB();
void SliderCB();
void Usage();
int retrieve_color();
 
FL_OBJECT *input_field;
FL_OBJECT *slider;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* widtools */

void main (int argc, char **argv)
{
    int i;
    int label_width;
    int form_x, form_y;
    char *label = NULL;
    
    FL_IOPT control;       
    int version, revision;

    Display *dpy;
    int root_h, root_w, junk;
    Window wjunk;
    
    /* Default options */
    int opBgColor       = FL_COL1;
    int opFgColor       = FL_BLACK;
    int opFnSize        = FL_NORMAL_SIZE;
    int opFnStyle       = FL_BOLD_STYLE;
    int opWidth         = -1;
    int opHeight        = -1;
    char *opTitle       = "widtools";
    int opExFlag        = -1;     
    int opButUpColor    = FL_COL1;
    int opButDownColor  = FL_MCOL;
    char *opOkName      = NULL;
    char *opCancelName  = NULL;
    char *opDismissName = NULL;
    int opPlacement     = FL_PLACE_CENTER;
             
             
    FL_FORM *main_form;
    FL_OBJECT *obj, *ret;
     
    /* dialog */
    char **discriminator;
    char *type;
    
    /* getfile */
    char *filter = "*";
    char *directory = ".";
    const char *selected_file;

    /* input */
    int input_class;
    int int_input    = 0;
    int float_input  = 0;
    int secret_input = 0;
    char *start_text = NULL;
    int cursorColor  = FL_BLUE;
         
    /* menu */
    int test_width;
    int button_width, button_height;
    int break_pt;
    char *menu_buttons[MAX_BUTTONS];
        
    /* showfile */
    FL_OBJECT *browser;
    char *file = NULL;
 
    /* slider */
    double lo     = 0.0;
    double hi     = 1.0;
    double step   = 0.01;
    int precision = 2;
    int init_flag = 0;
    double initial;
    double max_lo, max_hi, max_num;
    int pad;

    
    /* Make sure the distribution is consistent wrt version numbers */
    if (FL_INCLUDE_VERSION != fl_library_version(&version, &revision)) {
       fprintf(stderr,"*** Header/library version mismatch!\n");
       fprintf(stderr,"    Header: %d.%d\n", FL_VERSION, FL_REVISION);
       fprintf(stderr,"    Library: %d.%d\n", version, revision);
       exit(1);
    }
    

    /* Set some defaults and initialize the library */
    control.inputFontSize   = 12;
    control.browserFontSize = 12;
    control.buttonFontSize  = 12;
    control.labelFontSize   = 12;
    control.menuFontSize    = 12;
    control.borderWidth     = -2;
    fl_set_defaults(FL_PDInputFontSize|FL_PDBrowserFontSize|
        FL_PDButtonFontSize|FL_PDLabelFontSize|
        FL_PDMenuFontSize|FL_PDBorderWidth, &control);
    
    dpy = fl_initialize (&argc, argv, "Widtools", 0, 0);

    /* Get the root window size */
    XGetGeometry(dpy, XDefaultRootWindow(dpy),
        &wjunk, &junk, &junk, &root_w, &root_h, &junk, &junk);

    /* Parse options */
    if (argc == 1)
       Usage();

    for (i=0; i<argc; i++) {
             
        /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
        /* Executable options */
        
        if (strcmp(argv[i], "-dialog") == 0)
           opExFlag = 1;        
        if (strcmp(argv[i], "-getfile") == 0)
           opExFlag = 2;        
        if (strcmp(argv[i], "-input") == 0)
           opExFlag = 3;        
        if (strcmp(argv[i], "-menu") == 0)
           opExFlag = 4;        
        if (strcmp(argv[i], "-showfile") == 0) {
           opExFlag = 5;
           if ((strcmp(argv[i+1], "-dir") != 0) &&
               (strcmp(argv[i+1], "-filter") != 0))
              file = (char *) argv[i+1];
        }   
        if (strcmp(argv[i], "-slider") == 0)
           opExFlag = 6;        


        /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
        /* Global options */

        /* Help */
        if ((strcmp(argv[i], "-h") == 0) || (strcmp(argv[i], "-help") == 0)) {
           Usage();
           return;           
        }
             
        /* Placement */
        if (strcmp(argv[i], "-place") == 0) {
           if (strcmp(argv[i+1], "center") == 0) 
              opPlacement = FL_PLACE_CENTER;
           if (strcmp(argv[i+1], "mouse") == 0) 
              opPlacement = FL_PLACE_MOUSE;
        }
                        
        /* Background color */
        if (strcmp(argv[i], "-back") == 0)
           opBgColor = retrieve_color(argv[i+1]);

        /* Foreground color */
        if (strcmp(argv[i], "-fore") == 0)
           opFgColor = retrieve_color(argv[i+1]);
                      
        /* ButtonUp color */
        if (strcmp(argv[i], "-but_up") == 0)
           opButUpColor = retrieve_color(argv[i+1]);

        /* ButtonDown color */
        if (strcmp(argv[i], "-but_down") == 0)
           opButDownColor = retrieve_color(argv[i+1]);
 
        /* Font size and style */
        if (strcmp(argv[i], "-fn") == 0) {
           opFnSize  = retrieve_font_size(argv[i+1]);
           opFnStyle = retrieve_font_style(argv[i+2]);
        }
     
        /* Form width, height */
        if (strcmp(argv[i], "-width") == 0) 
           opWidth =  atoi(argv[i+1]);
        if (strcmp(argv[i], "-height") == 0) 
           opHeight =  atoi(argv[i+1]);

        /* Form title */
        if (strcmp(argv[i], "-title") == 0)
           opTitle = argv[i+1];

        /* Button names */
        if (strcmp(argv[i], "-okname") == 0)
           opOkName = argv[i+1];
        if (strcmp(argv[i], "-cancelname") == 0)
           opCancelName = argv[i+1];
        if (strcmp(argv[i], "-dismissname") == 0)
           opDismissName = argv[i+1];
        

        /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
        /* Specific options */

        /* dialog */
        if (strcmp(argv[i], "-type") == 0)
           type = argv[i+1];
        if (strcmp(argv[i], "-label") == 0) 
           label = argv[i+1];

        /* getfile */
        if (strcmp(argv[i], "-filter") == 0) 
           filter = argv[i+1];          
        if (strcmp(argv[i], "-dir") == 0) 
           directory = argv[i+1];

        /* input */
        if (strcmp(argv[i], "-int") == 0) 
           int_input = 1;
        if (strcmp(argv[i], "-float") == 0) 
           float_input = 1;
        if (strcmp(argv[i], "-secret") == 0) 
           secret_input = 1;
        if (strcmp(argv[i], "-text") == 0) 
           start_text = (char *) argv[i+1];
        if (strcmp(argv[i], "-cursor") == 0)
           cursorColor = retrieve_color(argv[i+1]);

        /* menu */

        /* showfile */
        if (strcmp(argv[i], "-file") == 0) 
           file = argv[i+1];

        /* slider */
        if (strcmp(argv[i], "-min") == 0)
           sscanf(argv[i+1], "%lf", &lo);
        if (strcmp(argv[i], "-max") == 0)
           sscanf(argv[i+1], "%lf", &hi);
        if (strcmp(argv[i], "-step") == 0)
           sscanf(argv[i+1], "%lf", &step);
        if (strcmp(argv[i], "-dec") == 0)
           sscanf(argv[i+1], "%d", &precision);
        if (strcmp(argv[i], "-init") == 0) {
           init_flag = 1;
           sscanf(argv[i+1], "%lf", &initial);
        }  

    } /* endfor i */


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
    /* dialog */ 

    if (opExFlag == 1) {


       /* discriminator holds the array of char pointers obtained from the
       ** #include of the pixmaps
       */
       if (strcmp(type, "error") == 0)
	  discriminator = error_xpm;
       else if (strcmp(type, "info") == 0)
	  discriminator = info_xpm;
       else if (strcmp(type, "message") == 0)
	  discriminator = message_xpm;	  
       else if (strcmp(type, "question") == 0)
	  discriminator = question_xpm;
       else if (strcmp(type, "warning") == 0)
	  discriminator = warning_xpm;
       else
 	  Usage();
 
       /* Get the string width, then size the form */
       if (label == NULL)
          label = type;
       label_width = fl_get_string_width(opFnStyle, opFnSize,
                     label, strlen(label));


       if (opWidth == -1) {
	  form_x = label_width + (2.0 * label_width * 0.10);
	  form_x = form_x + 50;
	  if (form_x < 200)
             form_x = 200;
       }
       else
	  form_x = opWidth;

       if (opHeight == -1)
	  form_y = 120;
       else
	  form_y = opHeight;   

       main_form = fl_bgn_form(FL_UP_BOX, form_x, form_y);

           obj = fl_add_box(FL_UP_BOX,
               0, 0, form_x, form_y, "");
           fl_set_object_color(obj, opBgColor, 0);
           
           obj = fl_add_pixmap(FL_NORMAL_PIXMAP, 
               10, 2, 50, 50, "");
           fl_set_pixmap_data(obj, discriminator);

           obj = fl_add_box(FL_NO_BOX, 
               (60 + form_x - label_width)/2, 15, label_width, 25, label);
               
           fl_set_object_lstyle(obj, opFnStyle);
           fl_set_object_lsize(obj, opFnSize);
           fl_set_object_lcol(obj, opFgColor);

           obj = fl_add_box(FL_NO_BOX, 
               0, 50, form_x, 10, "@DnLine"); 

           obj = fl_add_button(FL_NORMAL_BUTTON, 
               20, 80, 70, 25, "Ok"); 
           if (opOkName != NULL)
              fl_set_object_label(obj, opOkName);
           fl_set_object_lstyle(obj, opFnStyle);
           fl_set_object_lsize(obj, opFnSize);
           fl_set_object_lcol(obj, opFgColor);
           fl_set_object_color(obj, opButUpColor, opButDownColor);
           fl_set_object_callback(obj, DialogCB, 0);

           obj = fl_add_button(FL_NORMAL_BUTTON, 
               form_x - 70 - 20, 80, 70, 25, "Cancel"); 
           if (opCancelName != NULL)
              fl_set_object_label(obj, opCancelName);
           fl_set_object_lstyle(obj, opFnStyle);
           fl_set_object_lsize(obj, opFnSize);
           fl_set_object_lcol(obj, opFgColor);
           fl_set_object_color(obj, opButUpColor, opButDownColor);
           fl_set_object_callback(obj, DialogCB, 1);
 
       fl_end_form();

       fl_show_form(main_form, opPlacement, FL_FULLBORDER, opTitle);

       do {
	  ret = fl_do_forms();
       } while (ret != NULL);

       fl_hide_form(main_form);
       exit(0);

    } /* endif dialog */


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
    /* getfile */ 

    if (opExFlag == 2) {

         fl_set_fselector_title(opTitle);
         
	 selected_file = fl_show_fselector("Enter Filename:", 
             directory, filter, "");  
         
         /* If Cancel is selected, fl_show_fselector returns "(null)" */
         if (strcmp(selected_file, "(null)") == 0)
            selected_file = "";
            
	 printf("%s\n", selected_file);
       
         fl_finish();
         exit(0);

    } /* endif getfile */


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
    /* input */ 

    if (opExFlag == 3) {
  
       if (int_input == 1) {
          if (label == NULL)
             label = "Enter an Integer";
          input_class = FL_INT_INPUT;
       }   
       else if (float_input == 1) {
          if (label == NULL)
             label = "Enter a Float";
          input_class = FL_FLOAT_INPUT;
       }   
       else if (secret_input == 1) {
          if (label == NULL)
             label = "Enter Text";
          input_class = FL_SECRET_INPUT;
       }
       else {
          if (label == NULL)
             label = "Enter Text";
          input_class = FL_NORMAL_INPUT;
       }
             
 
       /* Get the label width, then size the form */
       label_width = fl_get_string_width(opFnStyle, opFnSize,
                     label, strlen(label));

       if (opWidth == -1) {
          form_x = label_width + (2.0 * label_width * 0.10);
          form_x = form_x + 50;
          if (form_x < 200)
             form_x = 200;
       }
       else
	  form_x = opWidth;

       if (opHeight == -1)
	  form_y = 120;
       else
	  form_y = opHeight;   
          
       main_form = fl_bgn_form(FL_UP_BOX, form_x, form_y);

           obj = fl_add_box(FL_UP_BOX,
               0, 0, form_x, form_y, "");
           fl_set_object_color(obj, opBgColor, 0);

           obj = fl_add_box(FL_NO_BOX, 
               (form_x - label_width)/2, 15, label_width, 25, label); 
           fl_set_object_lsize(obj, opFnSize);
           fl_set_object_lstyle(obj, opFnStyle);
           fl_set_object_lcol(obj, opFgColor);

           obj = fl_add_box(FL_NO_BOX, 
               0, 50, form_x, 10, "@DnLine"); 

           input_field = fl_add_input(input_class, 
               form_x * 0.05, 75, form_x * 0.9, 25, ""); 
           if (start_text != NULL)
              fl_set_input(input_field, start_text);
           fl_set_object_lsize(input_field, opFnSize);
           fl_set_object_lstyle(input_field, opFnStyle);
           fl_set_input_color(input_field, opFgColor, cursorColor);
           fl_set_input_return(input_field, FL_RETURN_END);
           fl_set_object_callback(input_field, InputCB, 0);

       fl_end_form();

       fl_show_form(main_form, opPlacement, FL_FULLBORDER, opTitle);

       do {
	  ret = fl_do_forms();
       } while (ret != NULL);

       fl_hide_form(main_form);
       exit(0);
 
    } /* endif input */


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
    /* menu */

    if (opExFlag == 4) {

       /* -label is a required option */  
       if (strcmp(argv[2], "-label") != 0)
          Usage();
                    
       /* Get the maximum string width, then size the form */
       label_width = fl_get_string_width(opFnStyle, opFnSize,
           label, strlen(label));
 
       for (i=4; i<argc; i++) {
       
           if (argv[i][0] == '-') {
              break_pt = i;
              break;
           }   
              
           menu_buttons[i-4] = argv[i];
                                                    
           test_width = fl_get_string_width(opFnStyle, opFnSize,
                	menu_buttons[i-4], strlen(menu_buttons[i-4]));
           if (test_width > label_width)
              label_width = test_width;

           break_pt = i+1;
       }
 
       button_width  = label_width + 20;
       button_height = 25;

       if (opWidth == -1) {
	  form_x = button_width + 20;
	  if (form_x < 200) {    
	     form_x = 200;
	     button_width = 180;
	  }
       }
       else {
	  form_x = opWidth;
	  button_width = opWidth - 20;
       }  

       if (opHeight == -1)
          form_y = (button_height * (break_pt - 4 + 1)) + 30 + 10;
       else
	  form_y = opHeight;   

       main_form = fl_bgn_form(FL_UP_BOX, form_x, form_y); 

           obj = fl_add_box(FL_UP_BOX,
               0, 0, form_x, form_y, "");
           fl_set_object_color(obj, opBgColor, 0);

           obj = fl_add_box(FL_NO_BOX, 
               10, 10, button_width, button_height, label);
           fl_set_object_lstyle(obj, opFnStyle);
           fl_set_object_lsize(obj, opFnSize);
           fl_set_object_lcol(obj, opFgColor);
              
           obj = fl_add_box(FL_NO_BOX, 
               0, 5 + button_height, form_x, 10, "@DnLine");  

           for (i=4; i<break_pt; i++) {

               obj = fl_add_button(FL_NORMAL_BUTTON, 
                     10, 10 + 20 + (button_height * (i - 3)), 
                     button_width, button_height, menu_buttons[i-4]); 
               fl_set_object_lstyle(obj, opFnStyle);
               fl_set_object_lsize(obj, opFnSize);
               fl_set_object_lcol(obj, opFgColor);
               fl_set_object_color(obj, opButUpColor, opButDownColor);

               fl_set_object_callback(obj, MenuCB, i-3);

           }

       fl_end_form();

       fl_show_form(main_form, opPlacement, FL_FULLBORDER, opTitle);

       do {
	  ret = fl_do_forms();
       } while (ret != NULL);

       fl_hide_form(main_form);
       exit(0);

    } /* endif menu */


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
    /* showfile */ 

    if (opExFlag == 5) {
    
       /* Form size */
       if (opWidth == -1)
	  form_x = 500;
       else
	  form_x = opWidth;   

       if (opHeight == -1)
	  form_y = 375;
       else
	  form_y = opHeight;   

       if (file == NULL) {
          fl_set_fselector_title(opTitle);
         
	  file = (char *) fl_show_fselector("Enter Filename:", 
              directory, filter, "");
              
          /* file is still NULL if Cancel is selected */
       }

       main_form = fl_bgn_form(FL_UP_BOX, form_x, form_y);

           obj = fl_add_box(FL_UP_BOX,
               0, 0, form_x, form_y, "");
           fl_set_object_color(obj, opBgColor, 0);

           browser = fl_add_browser(FL_NORMAL_BROWSER, 
               10, 10, form_x - 20, form_y - 50, ""); 
           fl_load_browser(browser, file);
           fl_set_browser_fontsize(browser, opFnSize);
           fl_set_browser_fontstyle(browser, opFnStyle);

           obj = fl_add_button(FL_NORMAL_BUTTON, 
               (form_x - 75)/2, form_y - 35, 75, 25, "Dismiss");
           if (opDismissName != NULL)
              fl_set_object_label(obj, opDismissName);
           fl_set_object_lstyle(obj, opFnStyle);
           fl_set_object_lsize(obj, opFnSize);
           fl_set_object_lcol(obj, opFgColor);
           fl_set_object_color(obj, opButUpColor, opButDownColor);
           fl_set_object_callback(obj, QuitCB, 0);

       fl_end_form();

       if (form_x > root_w) {
	  printf("showfile: Some browser lines too wide for display!\n");
	  form_x = root_w;
       }

       fl_set_form_size(main_form, form_x, form_y);

       if (strcmp(opTitle, "widtools") == 0)
          opTitle = file;
      
       if (file == NULL) {
          printf("showfile: file = NULL\n");
          return;
       }   
       else    
          fl_show_form(main_form, opPlacement|FL_FREE_SIZE, 
              FL_FULLBORDER, opTitle);

       do {
	  ret = fl_do_forms();
       } while (ret != NULL);

       fl_hide_form(main_form);
       exit(0);
 

    } /* endif showfile */


    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
    /* slider */ 

    if (opExFlag == 6) {
 
       if (label == NULL)
          label = "Select Value";


       /* Size the form */

       /* Get the absolute maximum number displayed */
       max_lo = fabs(lo);
       max_hi = fabs(hi);
       if (max_hi > max_lo)
	  max_num = max_hi;
       else
	  max_num = max_lo;

       /* Size the form based on the maximum number.  If number is greater
       ** than 10000, well...
       */
       if (max_num < 100) 
	  form_x = 300;
       else if (max_num < 1000)
	  form_x = 400;
       else if (max_num < 10000)
	  form_x = 450;
       else
	  form_x = 500;

       /* Add space for number of decimal points displayed */
       if (precision >= 2)
	  pad = (precision - 2) * 50; 
       else
	  pad = 0;

       form_x = form_x + pad;

       /* Get the label width, then resize the form if needed */
       label_width = fl_get_string_width(opFnStyle, opFnSize,
                     label, strlen(label));

       if (form_x < label_width + (2.0 * label_width * 0.10))
	  form_x = label_width + (2.0 * label_width * 0.10);

       form_y = 120;

       main_form = fl_bgn_form(FL_UP_BOX, form_x, form_y);

           obj = fl_add_box(FL_UP_BOX,
               0, 0, form_x, form_y, "");
           fl_set_object_color(obj, opBgColor, 0);

           slider = fl_add_valslider(FL_HOR_SLIDER, 
               form_x * 0.05, 15, form_x * 0.90, 25, label);

           fl_set_object_lstyle(slider, opFnStyle);
           fl_set_object_lcol(slider, opFgColor);
           fl_set_object_lsize(slider, opFnSize);
           fl_set_slider_bounds(slider, lo, hi);
           fl_set_slider_step(slider, step);
           fl_set_slider_precision(slider, precision);
           if (init_flag == 1)
              fl_set_slider_value(slider, initial);
           else
              fl_set_slider_value(slider, (hi-lo)/2.0);

           obj = fl_add_box(FL_NO_BOX, 
               0, 60, form_x, 10, "@DnLine");  

           obj = fl_add_button(FL_NORMAL_BUTTON,
               (form_x - 70) / 2, 80, 70, 25, "Dismiss");
           if (opDismissName != NULL)
              fl_set_object_label(obj, opDismissName);
           fl_set_object_callback(obj, SliderCB, 0);
           fl_set_object_lstyle(obj, opFnStyle);
           fl_set_object_lsize(obj, opFnSize);
           fl_set_object_lcol(obj, opFgColor);
           fl_set_object_color(obj, opButUpColor, opButDownColor);

       fl_end_form();

       fl_show_form(main_form, opPlacement, FL_FULLBORDER, opTitle);

       do {
	  ret = fl_do_forms();
       } while (ret != NULL);

       fl_hide_form(main_form);
       exit(0);

 
    } /* endif slider */


} /* END */



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int retrieve_color(char *input)
{
    int fl_color;
    
  
    if (strcmp(input, "black") == 0)
       fl_color = FL_BLACK;
    else if (strcmp(input, "red") == 0)
       fl_color = FL_RED;
    else if (strcmp(input, "green") == 0)
       fl_color = FL_GREEN;
    else if (strcmp(input, "yellow") == 0)
       fl_color = FL_YELLOW;
    else if (strcmp(input, "blue") == 0)
       fl_color = FL_BLUE;
    else if (strcmp(input, "cyan") == 0)
       fl_color = FL_CYAN;
    else if (strcmp(input, "magenta") == 0)
       fl_color = FL_MAGENTA;
    else if (strcmp(input, "white") == 0)
       fl_color = FL_WHITE;
    else if (strcmp(input, "tomato") == 0)
       fl_color = FL_TOMATO;
    else if (strcmp(input, "indianred") == 0)
       fl_color = FL_INDIANRED;
    else if (strcmp(input, "slateblue") == 0)
       fl_color = FL_SLATEBLUE;
    else if (strcmp(input, "darkgold") == 0)
       fl_color = FL_DARKGOLD;
    else if (strcmp(input, "palegreen") == 0)
       fl_color = FL_PALEGREEN;
    else if (strcmp(input, "orchid") == 0)
       fl_color = FL_ORCHID;
    else if (strcmp(input, "darkcyan") == 0)
       fl_color = FL_DARKCYAN;
    else if (strcmp(input, "darktomato") == 0)
       fl_color = FL_DARKTOMATO;
    else if (strcmp(input, "wheat") == 0)
       fl_color = FL_WHEAT;
    else if (strcmp(input, "col1") == 0)
       fl_color = FL_COL1;

    else if (strcmp(input, "gray") == 0)
       fl_color = FL_COL1;
    else if (strcmp(input, "grey") == 0)
       fl_color = FL_COL1;

    else if (strcmp(input, "mcol") == 0)
       fl_color = FL_MCOL;

    else if (strcmp(input, "darkgray") == 0)
       fl_color = FL_MCOL;
    else if (strcmp(input, "darkgrey") == 0)
       fl_color = FL_MCOL;

    else if (strcmp(input, "bcol") == 0)
       fl_color = FL_TOP_BCOL;
    else if (strcmp(input, "bottom_bcol") == 0)
       fl_color = FL_BOTTOM_BCOL;
    else if (strcmp(input, "left_bcol") == 0)
       fl_color = FL_LEFT_BCOL;
    else if (strcmp(input, "right_bcol") == 0)
       fl_color = FL_RIGHT_BCOL;
    else if (strcmp(input, "inactive_col") == 0)
       fl_color = FL_INACTIVE_COL;
    else {
       printf("Unknown color: %s\n", input);
       /* Return a default */
       return FL_COL1;
    }
    
    return fl_color;
 
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int retrieve_font_size(char *input)
{
    int fl_font_size;
 

    if (strcmp(input, "tiny") == 0)
       fl_font_size = FL_TINY_SIZE;
    else if (strcmp(input, "small") == 0)
       fl_font_size = FL_SMALL_SIZE;
    else if (strcmp(input, "normal") == 0)
       fl_font_size = FL_NORMAL_SIZE;
    else if (strcmp(input, "medium") == 0)
       fl_font_size = FL_MEDIUM_SIZE;
    else if (strcmp(input, "large") == 0)
       fl_font_size = FL_LARGE_SIZE;
    else if (strcmp(input, "huge") == 0)
       fl_font_size = FL_HUGE_SIZE;
    else {
       printf("Unknown font size: %s\n", input);
       /* Return a default */
       return FL_NORMAL_SIZE;
    }

    return fl_font_size;

}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int retrieve_font_style(char *input)
{
    int fl_font_style;


    if (strcmp(input, "normal") == 0)
       fl_font_style = FL_NORMAL_STYLE; 
    else if (strcmp(input, "bold") == 0)
       fl_font_style = FL_BOLD_STYLE; 
    else if (strcmp(input, "italic") == 0)
       fl_font_style = FL_ITALIC_STYLE; 
    else if (strcmp(input, "bolditalic") == 0)
       fl_font_style = FL_BOLDITALIC_STYLE; 
    else if (strcmp(input, "fixed") == 0)
       fl_font_style = FL_FIXED_STYLE; 
    else if (strcmp(input, "fixedbold") == 0)
       fl_font_style = FL_FIXEDBOLD_STYLE; 
    else if (strcmp(input, "fixeditalic") == 0)
       fl_font_style = FL_FIXEDITALIC_STYLE; 
    else if (strcmp(input, "fixedbolditalic") == 0)
       fl_font_style = FL_FIXEDBOLDITALIC_STYLE; 
    else if (strcmp(input, "times") == 0)
       fl_font_style = FL_TIMES_STYLE; 
    else if (strcmp(input, "timesbold") == 0)
       fl_font_style = FL_TIMESBOLD_STYLE; 
    else if (strcmp(input, "timesitalic") == 0)
       fl_font_style = FL_TIMESITALIC_STYLE; 
    else if (strcmp(input, "timesbolditalic") == 0)
       fl_font_style = FL_TIMESBOLDITALIC_STYLE; 
    else if (strcmp(input, "shadow") == 0)
       fl_font_style = FL_SHADOW_STYLE; 
    else if (strcmp(input, "engraved") == 0)
       fl_font_style = FL_ENGRAVED_STYLE; 
    else if (strcmp(input, "embossed") == 0)
       fl_font_style = FL_EMBOSSED_STYLE; 
    else {
         printf("Unknown font style: %s\n", input);
	/* Return a default */
	return FL_BOLD_STYLE;
    }

    return fl_font_style;

}



/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void DialogCB(FL_OBJECT *ob, long data)
{
    fl_finish();
    
    printf("%d\n", data);
    exit (0);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void InputCB(FL_OBJECT *ob, long data)
{
    printf("%s\n", fl_get_input(input_field));

    fl_finish();
    exit (0);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void MenuCB(FL_OBJECT *ob, long data)
{
    fl_finish();
    printf("%d\n", data);
    exit (0);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void QuitCB(FL_OBJECT *ob, long data)
{
    fl_finish();
    exit (0);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void SliderCB(FL_OBJECT *ob, long data)
{
    printf("%lf\n", fl_get_slider_value(slider));

    fl_finish();
    exit (0);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void Usage(void)
{
       int ans;
       
       
       printf("\n");
       printf("Global options:\n");
       printf("   -back 	Background color [gray]\n");
       printf("   -fore 	Foreground color [black]\n");
       printf("   -but_up	Button color - inactive [gray]\n");
       printf("   -but_down	Button color - active [darkgray]\n");
       printf("   -fn		Font size and style [normal bold]\n");
       printf("   -width	Form width (pixels)\n");
       printf("   -height	Form height (pixels)\n");
       printf("   -title	Form title [widtools]\n");
       printf("   -bw		Border width [-2]\n");
       printf("   -okname	Change name of \"Ok\" button\n");
       printf("   -cancelname	Change name of \"Cancel\" button\n");
       printf("   -dismissname	Change name of \"Dismiss\" button\n");
       printf("\n");
retry:
       printf("Available widtools:\n");
       printf("  1. dialog\n");
       printf("  2. getfile\n");
       printf("  3. input\n");
       printf("  4. menu\n");
       printf("  5. showfile\n");
       printf("  6. slider\n");
       printf("\n");
       printf("  7. Show available colors\n");
       printf("  8. Show available font styles\n");
       printf("  9. Show available font sizes\n");
       
       printf("Enter a number for specific information: ");
       
       ans = getchar();
       
       printf("\n");
       if ((char) ans == '1') {
          printf("widtools -dialog -type type -label label\n");
          printf("	type	error info message question warning (REQUIRED)\n");
          printf("	label	Text label for the dialog box\n");
       }
       else if ((char) ans == '2') {
          printf("widtools -getfile [-dir dir] [-filter filter]\n");
          printf("	dir	starting directory [\".\"]\n");
          printf("	filter	file filter [\"*\"]\n");
       }
       else if ((char) ans == '3') {
          printf("widtools -input [-int -float -secret] [-label label] [-text text] [-cursor color]\n");
          printf("	-int	Enter integers only\n");
          printf("	-float	Enter floats only\n");
          printf("	-secret	Input not shown on display\n");
          printf("	-label	text label\n");
          printf("	-text	Initial input text\n");
          printf("	-cursor	Cursor color [blue]\n");
       }
       else if ((char) ans == '4') {
          printf("widtools -menu -label label item_1 item_2 ... item_n\n");
          printf("	-label	menu label (REQUIRED)\n");
          printf("	item_n	menu selection n\n");
       }
       else if ((char) ans == '5') {
          printf("widtools -showfile [file] [-dir dir] [-filter filter]\n");
          printf("	file	file to display (if omitted, selection box is used)\n");
          printf("	dir	selection box starting directory [\".\"], if -file not set\n");
          printf("	filter	selection box file filter [\"*\"], if -file not set\n");
       }
       else if ((char) ans == '6') {
          printf("widtools -slider [-label label] [-min min] [-max max]\n"); 
          printf("                 [-step step] [-dec decimals] [-init init]\n");
          printf("	-label	Text label [Select Value]\n");
          printf("	-min	Minimum slider value [0.0]\n");
          printf("	-max	Maximum slider value [1.0]\n");
          printf("	-step	Step value [0.01]\n");
          printf("	-dec	Precision (number of decimal places) [2]\n");
          printf("	-init   Initial value [ (max-min)/2 ]\n");
       }
       else if ((char) ans == '7') {
          printf("  Colors:\n\n");
          printf("  black\t\t");
          printf("  red\t\t");
          printf("  green\n");
          printf("  blue\t\t");
          printf("  cyan\t\t");
          printf("  magenta\n");
          printf("  white\t\t");
          printf("  tomato\t");
          printf("  indianred\n");
          printf("  slateblue\t");
          printf("  darkgold\t");
          printf("  palegreen\n");
          printf("  orchid\t");
          printf("  darkcyan\t");
          printf("  darktomato\n");
          printf("  wheat\t\t");
          printf("  gray\t\t");
          printf("  grey\n");
          printf("  darkgray\t");
          printf("  darkgrey\t");
          printf("  col1\n");
          printf("  mcol\t\t");
          printf("  top_bcol\t");
          printf("  bottom_bcol\n");
          printf("  right_bcol\t");
          printf("  left_bcol\t");
          printf("  inactive_col\n");
       }          
       else if ((char) ans == '8') {
          printf("  Font Sizes:\n\n");
          printf("  tiny\t\t(8pt)\n");
          printf("  small\t\t(10pt)\n");
          printf("  normal\t(12pt)\n");
          printf("  medium\t(14pt)\n");
          printf("  large\t\t(18pt)\n");
          printf("  huge\t\t(24pt)\n");
       }
       else if ((char) ans == '9') {
          printf("  Font Styles:\n\n");
          printf("  normal\t");
          printf("  bold\t\t\t");
          printf("  italic\n");
          printf("  bolditalic\t");
          printf("  fixed\t\t\t");
          printf("  fixedbold\n");
          printf("  fixeditalic\t");
          printf("  fixedbolditalic\t");
          printf("  times\n");
          printf("  timesbold\t");
          printf("  timesitalic\t\t");
          printf("  timesbolditalic\n");
          printf("  shadow\t");
          printf("  engraved\t\t");
          printf("  embossed\n");
       }
       else {
          printf("\n\n");
          goto retry;
       }   
       
       exit (0);

}
